// components/v-stepper/index.js
/**
 * NumberBox 数字输入框
 * @description 带加减按钮的数字输入框
 * @tutorial https://ext.dcloud.net.cn/plugin?id=31
 * @property {Number} value 输入框当前值
 * @property {Number} min 最小值
 * @property {Number} max 最大值
 * @property {Number} step 每次点击改变的间隔大小
 * @property {String} background 背景色
 * @property {String} color 字体颜色（前景色）
 * @property {Boolean} disabled = [true|false] 是否为禁用状态
 * @property {String} stepTips 进步倍数纠正提示文案
 * @event {Function} change 输入框值改变时触发的事件，参数为输入框当前的 value
 * @event {Function} focus 输入框聚焦时触发的事件，参数为 event 对象
 * @event {Function} blur 输入框失焦时触发的事件，参数为 event 对象
 */
Component({
  lifetimes: {
    attached: function() {
      // 在组件实例进入页面节点树时执行
    },
    detached: function() {
      // 在组件实例被从页面节点树移除时执行
    },
  },
  /**
   * 组件的属性列表
   */
  properties: {
    value: {
      type: [Number, String],
      default: 1
    },
    min: {
      type: Number,
      default: 0
    },
    max: {
      type: Number,
      default: 100
    },
    step: {
      type: Number,
      default: 1
    },
    background: {
      type: String,
      default: '#fff'
    },
    color: {
      type: String,
      default: '#000'
    },
    disabled: {
      type: Boolean,
      default: false
    },
    stepTips: {
      type: String,
      default: ''
    },
    disableInput: {
      type: Boolean,
      default: false
    }
  },

  /**
   * 组件的初始数据
   */
  data: {
    inputValue: 0
  },

  /**
   * 组件的方法列表
   */
  methods: {
    // 输入完成数据预处理
    // input 和 按钮都触发
    preHandler(count) {
      // -. 小数转整数
      count = Math.ceil(count);
      // -. 取3的倍数
      if (count % this.data.step != 0) {
        wx.showToast({
          title: this.data.stepTips,
          icon: 'none',
          duration: 1500
        })
        count = Math.ceil(count / this.data.step) * this.data.step;
      }
      // -. 判断是否大于最大值
      if (count > this.data.max) {
        count = this.data.max;
      }
      // -. 判断是否小于最小值
      if (count < this.data.min) {
        count = this.data.min;
      }
      this.setData({
        inputValue: count
      });
      return count;
    },
    /**
     * 加减按钮
     * @param {*} type
     */
    _calcValue(e) {
      const type = e.currentTarget.dataset.type;
      let value = this.data.inputValue;
      // 判断是否禁用
      if (this.data.disabled) {
        return;
      }
      // 加减库存
      if (type === 'minus') {
        if (value <= this.data.min) {
          return;
        }
        value -= this.data.step;
      } else if (type === 'plus') {
        if (value >= this.data.max) {
          return;
        } 
        value += this.data.step;
      }
      value = this.preHandler(value);
      this.triggerEvent('change', value);
    },
    /**
     * 输入框失去焦点
     * @param {*} event
     */
    _onBlur(event) {
      let count = event.detail.value;
      count = this.preHandler(count);
      this.triggerEvent('change', +count);
    },
    _onFocus(event) {
      this.triggerEvent('focus', event);
    },
    onInput(e) {
      this.setData({
        inputValue: e.detail.value
      });
      this.triggerEvent('change', +inputValue);
    },
  },
  observers: {
    value(val) {
      const inputValue = this.data.inputValue + val;
      this.setData({inputValue});
    }
  }
});
